using System;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace DataBinding.DataAccess
{
    public class DatabaseConnection : DataBinding.DataAccess.IDatabaseConnection
    {
        private const string DATABASE_CONNECTION_CONFIG_KEY = "DatabaseConnection";
        
        private ConnectionInformation connectionInformation;
        private static object mutex = new object();
        private IDbConnection underlyingConnection;
        
        
        private class ConnectionInformation
        {
            private DbProviderFactory providerFactory;
            private ConnectionStringSettings settings;


            public ConnectionInformation(ConnectionStringSettings settings)
            {
                this.providerFactory = DbProviderFactories.GetFactory(settings.ProviderName);                
                this.settings = settings;
            }
            
            public IDbConnection CreateConnection()
            {
                IDbConnection connection = providerFactory.CreateConnection();
                connection.ConnectionString = settings.ConnectionString;
                return connection;
            }
        }
        
        public DatabaseConnection()
        {
            InitializeConnectionInformation();
            OpenUnderlyingConnection();
        }

        private void OpenUnderlyingConnection()
        {
            underlyingConnection = connectionInformation.CreateConnection();
            underlyingConnection.Open();
        }

        public void Close()
        {
            underlyingConnection.Close();
        }

        private void InitializeConnectionInformation()
        {
            lock (mutex)
            {
                if (connectionInformation == null)
                {
                    connectionInformation = new ConnectionInformation(ConfigurationManager.ConnectionStrings[DATABASE_CONNECTION_CONFIG_KEY]);
                }
            }
        }

        public void Dispose()
        {
            Close();
            underlyingConnection.Dispose();
            underlyingConnection = null;
        }


        public IDbConnection UnderlyingConnection
        {
            get { return underlyingConnection; }
        }

        public IDbCommand CreateCommandFor(string dynamicSqlExpression)
        {
            IDbCommand command = underlyingConnection.CreateCommand();
            command.CommandType = CommandType.Text;
            command.CommandText = dynamicSqlExpression;
            return command;
        }
    }
}